# Nebula Graph 查询语言 (nGQL)

## nGQL 概览

`nGQL` 是一种声明型的文本查询语言，目前尚在开发中。新版本会添加更多功能并优化已有功能。未来的语法规范可能会与现行的不一致。
## 目标

- 易学
- 易用
- 专注线上查询，同时为离线计算提供基础

## 特点

- 类 SQL，易学易用
- 可扩展
- 大小写不敏感
- 支持图遍历
- 支持模式匹配
- 支持聚合运算
- 支持图计算
- 支持分布式事务（开发中）
- 无嵌入支持组合语句，易于阅读

## 术语

- **图空间**：物理隔离的不同数据集
- **标签** ：拥有一种或多种属性
  - 每个标签都有一个人类可读的名称，并且每个标签内部都会分配一个 32 位的整数
  - 每个标签与一个属性列表相关联，每个属性都有一个名称和类型
  - 标签之间可存在依赖关系作为约束。例如，如果标签 S 依赖于标签 T，则除非标签 T 存在，否则标签 S 无法存在。
- **节点** ：图数据中代表实体的点
  - 每个节点都有一个唯一的 64 位（有符号整数）ID (**VID**)
  - 一个节点可以拥有多个**标签**
- **边**：节点之间的联系称为边
  - 每条边由唯一数组 **<src_vid, dst_vid, edge_type, rank>** 标识
  - ***边类型***是人类可读的字符串，并且每条边内部都会分配一个 32 位的整数。边类型决定边上的属性（模式）
  - ***边权重*** 是用户分配的不可变的 64 位带符号整数，决定两个顶点之间的边顺序。等级值较高的边排名靠前。如未指定，则默认等级值为零。
  - 每条边只能有一种类型
- **路径** : 多个节点与边的**非分支**连接
  - 路径长度为该路径上的边数，比节点数少 1
  - 路径可由一系列节点，边类型及权重表示。一条边是一个长度为 1 的特殊路径

```plain
 <vid, <edge_type, rank>, vid, ...>
```

## 查询语言规则概览

> 不熟悉 BNF 可跳过本节

### 总览

- 整套语句可分为三部分：**查询**，**更改**，**管理**
- 每条语句均可返回一个数据集，第个数据集均包含一个 schema 和多条数据

### 语句组合

- 语句组合有两种方式：
  - 语句可使用管道函数 "**|** 连接，前一条语句返回的结果可作为下一条语句的查询条件
  - 支持使用 "**;**" 批量输入多条语句，批处理时返回最后一条语句结果

### 数据类型

- 简单类型： **vid**, **double**, **int**, **bool**, **string**, **timestamp**
 <!-- **float**,**path**, **year**, **month** (year/month), **date**, **datetime** -->
  - **vid** ： 64 位有符号整数，用来表示点 ID
- 简单类型列表，如： **integer[]**, **double[]**, **string[]**
- **Map**: 键值对列表。键类型必须为 **字符**，值类型必须与给定 map
- **Object** (未来版本支持): 键值对列表。键类型必须为**字符**，值可以是任意简单类型
- **Tuple List**: *只适用于返回值*。由元数据和数据（多行）组成 。元数据包含列名和类型。

### 类型转换

- -一个简单的类型值可以隐式转换为列表
- -列表可以隐式转换为单列元组列表
  - "<type\>\_list" 可用来表示列名

### 常用 BNF

<simple\_type> ::= **vid** | **integer** | **double** | **float** | **bool** | **string** | **path** | **timestamp** | **year** | **month** | **date** | **datetime**

<composite_type> ::=

<type\> ::= <simple_type> | <composite\_type>

<vid\_list> ::= **vid** (, **vid**)\* | "{" **vid** (, **vid**)\* "}"

<label\> ::= \[:alpha\] ([:alnum:] | "\_")\*

<underscore\_label> ::= ("\_")* <label\>

<field\_name> ::= <label\>

<field\_def\_list> ::= <field\_def> (, <field\_def>)\*

<field\_def> ::= <field_name>:<type\>

<tuple\_list\_decl> ::= <tuple\_schema> ":" <tuple\_data>

<tuple\_schema> ::= <field\_def\_list>

<tuple\_data> ::= <tuple\> (, <tuple\>)\* | "{" <tuple\> (, <tuple\>)\* "}"

<tuple\> ::= "(" **VALUE** (, **VALUE**)\* ")"

<var\> ::= "$" <label\>

### 查询语句

#### 选择图空间

nebula 支持多图空间。不同图空间的数据彼此隔离。在进行查询前，需指定图空间。

<span style="color:blue">**USE**</span>  <graphspace_name>

#### 返回数据集

返回单个值或数据集

<span style="color:blue">**RETURN**</span> <return\_value\_decl>

<return\_value\_decl> ::= **vid** | <vid\_list> | <tuple\_list\_decl> | <var\>

#### 创建标签

使用以下语句创建**新**标签

<span style="color:blue">**CREATE TAG**</span> <tag\_name> (<prop\_def\_list>)

<tag\_name> ::= <label\> <br>
<prop\_def\_list> ::= <prop\_def>+ <br>
<prop\_def> ::= <prop\_name>,<type\> <br>
<prop\_name> ::= <label\> <br>

#### 创建边类型

使用以下语句创建**新**的边类型

<span style="color:blue">**CREATE EDGE**</span> <edge\_type\_name> (<prop\_def\_list>)

<edge\_type\_name> := <label\>


#### 插入节点

使用以下语句插入一个或多个节点

<span style="color:blue">**INSERT VERTEX**</span> [<span style="color:blue">**NO OVERWRITE**</span>] <tag\_list> <span style="color:blue">**VALUES**</span> <vertex\_list> <br/>

<tag\_list> ::= <tag\_name>(<prop\_list>) (, <tag\_name>(<prop\_list>))\* <br/>
<vertex\_list> ::= <vertex\_id>:(<prop\_value\_list>) (, <vertex\_id>:(<prop\_value\_list>))\* <br/>
<vertex\_id> ::= **vid** <br/>
<prop\_list> ::= <prop\_name> (, <prop\_name>)\* <br/>
<prop\_value\_list> ::= **VALUE** (, **VALUE**)\* <br/>

#### 插入边

使用以下语句插入一条或多条边

<span style="color:blue">**INSERT EDGE**</span> [<span style="color:blue">**NO OVERWRITE**</span>] <edge\_type\_name> [(<prop\_list>)] <span style="color:blue">**VALUES**</span> (<edge\_value>)+

edge\_value ::= <vertex\_id> -> <vertex\_id> [@ <weight\>] : <prop\_value\_list>

#### 更新节点

使用以下语句更新节点

<span style="color:blue">**UPDATE VERTEX**</span>  <vertex\_id>
<span style="color:blue">**SET**</span> \<update\_decl\>
[<span style="color:blue">**WHERE**</span> <conditions\>]
[<span style="color:blue">**YIELD**</span> <field\_list>]

<update\_decl> ::= <update\_form1> | <update\_form2> <br>
<update\_form1> ::= <prop\_name> = <expression\> {,<prop\_name> = <expression\>}+ <br>
<update\_form2> ::= (<prop\_list>) = (<value\_list>) | (<prop\_list>) = <var\> <br>

#### 更新边

使用以下语句更新边

<span style="color:blue">**UPDATE EDGE**</span>  <vertex\_id> -> <vertex\_id> [@<weight\>] <span style="color:blue">**OF**</span> <edge\_type>
<span style="color:blue">**SET**</span> <update\_decl>
[<span style="color:blue">**WHERE**</span> <conditions\>]
[<span style="color:blue">**YIELD**</span> <field\_list>]

#### 图遍历

根据指定条件遍历给定节点的关联节点，返回节点 ID 列表或数组

<span style="color:blue">**GO**</span>
[<steps\_decl> <span style="color:blue">**STEPS**</span>]
<span style="color:blue">**FROM**</span> <data\_set\_decl>
[<span style="color:blue">**OVER**</span> [<span style="color:blue">**REVERSELY**</span>] <edge\_type\_decl>]
[<span style="color:blue">**WHERE**</span> <filter\_list>]
[<span style="color:blue">**YIELD**</span> <field\_list>]

<steps\_decl> ::= **integer** | **integer** <span style="color:blue">**TO**</span> **integer** | <span style="color:blue">**UPTO**</span> **integer** <br>
<data\_set\_decl> ::= [data\_set] [[<span style="color:blue">**AS**</span>] <label\>]<br/>
<data\_set> ::= **vid** | <vid\_list> | <tuple\_list\_decl> | <var\><br/>
<edge\_type\_decl> ::= <edge\_type\_list> [<span style="color:blue">**AS**</span> <label\>]
<edge\_type\_list> ::= <edge\_type> {, <edge\_type>}\* <br>
<edge\_type> ::= <label\> <br>

<filter\_list> ::= <filter\> {<span style="color:blue">**AND**</span> | <span style="color:blue">**OR**</span> <filter\>}\* <br>
<filter\> ::= <expression\> <span style="color:blue">**>**</span> | <span style="color:blue">**>=**</span> | <span style="color:blue">**<**</span> | <span style="color:blue">**<=**</span> | <span style="color:blue">**==**</span> | <span style="color:blue">**!=**</span> <expression\> | <expression\> <span style="color:blue">**IN**</span> <value\_list\> <br>
<field\_list> ::= <return\_field> {, <return\_field>}\* <br>
<return\_field> ::= <expression\> [<span style="color:blue">**AS**</span> <label\>] <br>

<span style="color:blue">**WHERE**</span> 语句仅适用于最终返回结果，对中间结果不适用。

跳过 <span style="color:blue">**STEP[S]**</span> 表示 **一步**

从起始点出发一跳，遍历所有满足<span style="color:blue">**WHERE**</span> 语句的关联点，只返回满足 <span style="color:blue">**WHERE**</span> 语句的结果。

多跳查询时，<span style="color:blue">**WHERE**</span> 语句只适用于最终结果，对中间结果不适用。例如：

```SQL
GO 2 STEPS FROM me OVER friend WHERE birthday > "1988/1/1"
```

以上语句查询所有生日在 1988/1/1 之后的二度好友。

又例如：

```SQL
GO UPTO 3 STEPS FROM me OVER friend WHERE birthday > "1988/1/1/"
```

以上语句查询所有生日在 1988/1/1 之后的好友，最多查询为 3 度。如果只查询到一条，则会全部返回。如果没有满足条件的数据，则查询 2 度好友，如果还是没有则查询 3 度。

以下语句查询生日在 1988/1/1 之后的3 度 至 5 度好友。

```SQL
GO 3 TO 5 STEPS FROM me OVER friend WHERE birthday > "1988/1/1/"
```

#### 搜索

以下语句对满足筛选条件的节点或边进行搜索。

<span style="color:blue">**FIND VERTEX**</span>
<span style="color:blue">**WHERE**</span> <filter\_list>
[<span style="color:blue">**YIELD**</span> <field\_list>]

<span style="color:blue">**FIND EDGE**</span>
<span style="color:blue">**WHERE**</span> <filter\_list>
[<span style="color:blue">**YIELD**</span> <field\_list>]

<!-- #### 模式匹配

对以下语句进行模式匹配，并返回数组列表或路径

<span style="color:blue">**MATCH**</span>
<regular\_path\_expression>
[<span style="color:blue">**FROM**</span> <data\_set\_decl>]
[<span style="color:blue">**WHERE**</span> <filter\_list>]
[<span style="color:blue">**YIELD**</span> <field\_list>] -->

### 属性关联

属性关联很常见，如 <span style="color:blue">**WHERE**</span> 语句和 <span style="color:blue">**YIELD**</span> 语句。nGQL 采用如下方式定义属性关联：

<property_ref> ::= <object\> "." <prop\_name> <br>
<object\> ::= <alias\_name> | <alias\_with\_tag> | <var\> <br>
<alias\_name> ::= <label\> <br>
<alias\_with\_tag> ::= <alias\_name> '[' <tag\_name> "]" <br>

<var\> 以 "$" 开始，特殊变量有两类：$- 和 $$。

$- 为输入值， $$ 为目标值。

所有属性名以字母开头。个别系统属性以 "\_" 开头。  "\_" 保留值。

### 内建属性

\_id : 节点 ID
\_type : 边类型
\_src : 边起始点 ID
\_dst : 边终点 ID
\_rank : 权重值
