# Quick Start

## Docker 方式

试用 nebula 最简单的方式是使用 Docker。开始试用前，请确保：

- 已安装最新版的 [Docker](https://docs.docker.com/)。
- 从 [Nebula Docker Hub](https://hub.docker.com/r/vesoft/nebula-graph) 获取最新的 nebula 镜像。如未获取，请使用如下命令获取：

```sh
> docker pull vesoft/nebula-graph:nightly
```

### 启动服务

`nebula` 镜像下载完成后，运行容器：

```sh
> docker run -it vesoft/nebula-graph:nightly /bin/bash
```

进入容器后，默认在根目录下 `/` ，切换到 `Nebula` 主目录:

```sh
> cd /usr/local/nebula/
```

启动 metad，storaged 和 graphd 服务:

```sh
> scripts/nebula.service start all
```

查看服务状态:

```sh
> scripts/nebula.service status all
```

### 连接 Nebula Graph

```sh
> bin/nebula -u=user -p=password
```

- -u 为用户名，默认值为 `user`
- -p 为密码，用户 `user` 的默认密码为 `password`

此外，如果 console (`bin/nebula`) 与 `graphd` 不在同台服务器上，你需要指定 `graphd` 的 ip 和 port

```sh
> bin/nebula -u=user -p=password --addr={graphd ip} --port={graphd port}
```

在部署过程遇到的任何问题，欢迎前往 [GitHub](https://github.com/vesoft-inc/nebula/issues) 向社区提 issue。

## 创建图数据

本节介绍如何构建图数据并进行查询。本示例基于下图构建：

![Untitled Diagram (1)](https://user-images.githubusercontent.com/51590253/60649144-0774c980-9e74-11e9-86d6-bad1653e70ba.png)

示例数据有三种类型的标签（_course_， _building_，_student_），两种类型的边（_select_ 和 _like_）。

### 创建图空间

Nebula 中的图存储于 **SPACE** 中，每个 Space 是一个物理隔离的空间。首先，需要创建一个 Space ，并使用该 Space 以完成之后的操作。

列出已有的 Space：

```SQL
nebula> SHOW SPACES;
```

创建一个名为 myspace_test2 的新 Space：

```SQL
nebula> CREATE SPACE myspace_test2(partition_num=1024, replica_factor=1);
-- 使用这个 space
nebula> USE myspace_test2;
```

- `replica_factor` 用来指定集群副本数。
- `partition_num` 用来指定一个副本中的分区数量。

### 定义图数据 Schema

使用 `CREATE TAG` 语句定义带有标签类型和属性列表的标签：

```SQL
nebula> CREATE TAG course(name string, credits int);
nebula> CREATE TAG building(name string);
nebula> CREATE TAG student(name string, age int, gender string);
```

使用 `CREATE EDGE` 语句定义边类型：

```SQL
nebula> CREATE EDGE like(likeness double);
nebula> CREATE EDGE select(grade int);
```

查看上述创建的标签和边类型：

```SQL
-- 查看标签列表
nebula> SHOW TAGS;
-- 查看边类型列表
nebula> SHOW EDGES;
```

查看标签或边类型的属性：

```SQL
-- 查看标签的属性
nebula> DESCRIBE TAG student;
-- 查看边类型的属性
nebula> DESCRIBE EDGE like;
```

### 插入数据

根据上图，插入相应的点和边：

```SQL
-- 插入点
nebula> INSERT VERTEX student(name, age, gender) VALUES 200:("Monica", 16, "female");
nebula> INSERT VERTEX student(name, age, gender) VALUES 201:("Mike", 18, "male");
nebula> INSERT VERTEX student(name, age, gender) VALUES 202:("Jane", 17, "female");
nebula> INSERT VERTEX course(name, credits),building(name) VALUES 101:("Math", 3, "No5");
nebula> INSERT VERTEX course(name, credits),building(name) VALUES 102:("English", 6, "No11");

-- 插入边
nebula> INSERT EDGE select(grade) VALUES 200 -> 101:(5);
nebula> INSERT EDGE select(grade) VALUES 200 -> 102:(3);
nebula> INSERT EDGE select(grade) VALUES 201 -> 102:(3);
nebula> INSERT EDGE select(grade) VALUES 202 -> 102:(3);
nebula> INSERT EDGE like(likeness) VALUES 200 -> 201:(92.5);
nebula> INSERT EDGE like(likeness) VALUES 201 -> 200:(85.6);
nebula> INSERT EDGE like(likeness) VALUES 201 -> 202:(93.2);
```

### 查询示例

Q1. 查询点 201 喜欢的点：

```SQL
nebula> GO FROM 201 OVER like;
=============
| like._dst |
=============
| 200       |
-------------
| 202       |
-------------
```

Q2. 查询点 201 喜欢的点，并筛选出年龄大于 17 岁的点，并返回其姓名，年龄，性别，将其重全名为 Friend，Age，Gender。

```SQL
nebula> GO FROM 201 OVER like WHERE $$.student.age >= 17 YIELD $$.student.name AS Friend, \
  $$.student.age AS Age, $$.student.gender AS Gender;
=========================
| Friend | Age | Gender |
=========================
|   Jane |  17 | female |
-------------------------
```

说明: 可以使用反斜杠(`\`)来换行

- `YIELD` 用来指定返回信息。
- `$^` 为起始点。
- `$$` 为目标点。

Q3. 查询点201喜欢的点选择了哪些课程和其对应年级。

```SQL
-- 使用管道
nebula> GO FROM 201 OVER like yield like._dst as id \
  | GO FROM $-.id OVER select YIELD $^.student.name AS Student,\
  $$.course.name AS Course, select.grade AS Grade;
=============================
| Student |  Course | Grade |
=============================
|  Monica |    Math |     5 |
-----------------------------
|  Monica | English |     3 |
-----------------------------
|    Jane | English |     3 |
-----------------------------

-- 使用临时变量
nebula> $var=GO FROM 201 OVER like yield like._dst as id;\
  GO FROM $var.id OVER select YIELD $^.student.name AS Student,\
  $$.course.name AS Course, select.grade AS Grade;
=============================
| Student |  Course | Grade |
=============================
|  Monica |    Math |     5 |
-----------------------------
|  Monica | English |     3 |
-----------------------------
|    Jane | English |     3 |
-----------------------------
```

说明:

- `|` 表示管道操作，前一个子查询的结果传递给后一个子查询。
- `$-` 表示输入流。

第二种方法使用了用户定义变量 `$var`，此变量可以在整个复合语句内使用。
