# Storage Design

## Abstract

This document is to give an introduction on the storage design of the graph database Nebula Graph.

The Storage Service of nebula is composed of two parts, one is `Meta Service` that stores the meta data, the other is `Storage Service`that stores the data. The two services are in two independent processes. The data and deployment are separated as well. But their architectures are almost the same.

## Architecture

---
![image](https://user-images.githubusercontent.com/42762957/67537389-c26bf000-f70d-11e9-9f9d-013cff70209a.png)

Pict.1 The Architecture of Storage Service

---

As shown in Picture One, there are three layers in Storage Service. The bottom layer is the local storage engine, providing `get`, `put`, `scan` and `delete` operations on local data. The related interfaces are in `KVStore/KVEngine.h` and users can develop their own local store plugins based on their needs. Currently, nebula provides store engine based on RocksDB.

Above the local storage engine is the consensus layer that implements multi group raft. Each partition corresponding to a Raft group, is for the data sharding. Currently, nebula uses `hash` to shard data. When creating a space, users need to specify the partition number. Once set, partition number can not be changed. Generally, the partition number should meet the scale out needs in the future.

Above the consensus layer is the storage interfaces that defines a set of APIs that are related with graph. These API requirements are translated into a set of kv operations targeting at the corresponding partition. It is this layer that makes the storage service a real graph storage, otherwise it's just a kv storage. Nebula doesn't separate kv as an independent service as graph query involves a lot of calculation that use schema, which is not existed in kv layer. Such architecture makes pushing down more easily.

## Schema & Partition

Nebula stores the vertices, edges and properties. Efficient property filtering is critical for a Graph Database.

Nebula uses tags to indicate vertices types. One vertexID can relate with multiple tags, and each tag has its own properties. In kv storage, we use `vertexID + TagID` to represent key, and the related properties are placed behind value after encoding, the format is shown in Picture Two:

---
![image](https://user-images.githubusercontent.com/42762957/67537434-f9420600-f70d-11e9-91f4-1359ef3c82b9.png)

Pict.2 Vertex Key Format

---

- `Type`: one byte, used to indicate key type, current types are data, index, system, etc.
- `Part ID`: three bytes, used to indicate the sharding partition. This field can be used to **scan the partition data based on the prefix when re-balancing the partition**
- `Vertex ID`: four bytes, used to indicate vertex ID
- `Tag ID`: four bytes, used to relate tag
- `Timestamp`: eight bytes, not available to users, used in MVCC in the future

Each edge in Nebula Graph is modeled as two independent key-values. One is stored in the same partition as the source vertex, namely out-edge. The other one is stored in the same partition as the destination vertex, namely in-edge. Generally, out-key and in-key are stored in different partitions.

There may be multiple edge types or multiple edges with the same edge type between two vertices. For example, define an edge type 'transfer', user A may transfer money to user B multiple times, thus a field `rank` is added to distinguish the transfer records between the two. Edge key format shown as Picture Three:

---
![image](https://user-images.githubusercontent.com/42762957/67537454-11b22080-f70e-11e9-9f9a-787091c60982.png)

Pict.3 Edge Key Format

---

- `Type`: one byte, used to indicate key type, current types are data, index, system, etc.
- `Part ID`: three bytes, used to indicate the sharding partition. This field can be used to **scan the partition data based on the prefix when re-balancing the partition**
- `Vertex ID`: four bytes, used to indicate source vertex ID in out-edge, dest vertex ID in in-edge
- `Edge Type`: four bytes, used to indicate edge type, greater than zero means out-edge, less than zero means in-edge
- `Rank`: four bytes, used to indicate multiple edges in one edge type. Users can set the filed based on needs, it can store weight such as _transaction time_, _transaction number_ and _weight of a certain order_
- `Timestamp`: eight bytes, not available to users, used in MVCC in the future

If the value of an edge type is greater than zero, the corresponding edge key format is shown in Picture Four; otherwise if the value less than zero, the corresponding edge key format is shown in Picture Five.

---
![image](https://user-images.githubusercontent.com/42762957/67537498-3efece80-f70e-11e9-9d41-0af45cdff104.png)

Pict.4 Out-key format

---

![image](https://user-images.githubusercontent.com/42762957/67537516-5047db00-f70e-11e9-9760-282b66c877f4.png)

Pict.5 In-key format

---

The properties of vertices and edges are stored in the values of the corresponding kv pairs after encoding. As a strong typed database, Nebula needs get the schema information in the Meta Service before decoding. Besides, when encoding properties, schema version is added to support online schema change.

Nebula shards data through `modulo operation` on vertex ID. All the _out-keys_, _in-keys_ and _tag data_ are placed in the same partition through `modulo operation` on vertex ID. In this way, query efficiency is increased dramatically. For on-line queries, the most common operation is to do Breadth-First-Search (BFS) expansion from a vertex, thus it's the basic operation to get a vertex's in-edge or out-edge, and such operation's performance determines the traverse's performance. In BFS, there are cases filtering on some properties, Nebula guarantees the operation efficiency by putting properties and vertices and edges together. At present, most graph databases verify their efficiency with Graph 500 or Twitter data set, which are, of no eloquence for properties are not included. While most cases use property graphs and BFS needs a lot of filtering as well.

## KVStore

Nebula develops its own kv store to meet the performance needs:

- **High performance** pure kv
- **Provided as a library**, as a strong typed database, the performance of pushing down is the key of the performance of Nebula
- **Strong data consistency**, determined by the distribution system
- **Written in C++**  

For users that are not sensitive to performance or unwilling to migrate data, Nebula provides plugins for the whole kv store, builds the storage service on the third party kv store. Currently, Nebula provides HBase plugin.

With the RocksDB as the local storage engine, Nebula supports self management of multiple hard disks to take full use of the parallel property. All users need to do is to config multiple data directories.  Nebula manages the distributed kv store in a unified scheduling way with meta service. All the partition distribution data and current machine status can be found in the meta service. Users can input commands in the console to add or remove machines to generate and execute a balance plan in meta service.

Nebula provides its own WAL mode to customize the WAL. Each partition owns its WAL to catch up data without WAL split to guarantee efficiency. Besides, Nebula defines a Command Log class to conduct special operations. These logs have no real data and only used to inform all replicas to execute certain operations with raft. What's more, Nebula also provides a log class to conduct atomic operation on partitions. For example, the CAS and the read-modify-write which fully uses raft serialization.

One Nebula kv store cluster supports multiple spaces, and each space is set with its own partition number and replica number. Different spaces are isolated physically and spaces in the same clusters supports different storage engine and sharding strategy.

## Raft

This part gives some details on how raft works for Nebula.

### Multi Raft Group

As the log IDs must be serial, almost all the implementations adapt Multi Raft Group to solve this problem. Therefore, partition number almost determines the performance of the Raft Group,but it doesn't mean the more partitions, the better the performance. Each raft group stores a set of state information and has its own WAL file, thus more partitions requires more costs. Also, when there are too many Partitions, the batch operation is meaningless if the load is not high enough. For example, a single online system with 10,000 tps, if its partition number is greater than ten thousand, the tps of each partition per second is likely only 1, so the batch operation loses its meaning and increases CPU cost.

There are two key points to achieve the Multi Raft Group. **The first is to share the transport layer**, because each Raft Group needs to send messages to the corresponding peers. If the transport layer cannot be shared, the connection costs are huge. **Second is the threading model**, multi Raft Group must share a set of thread pools, otherwise will end with too many threads in the system, resulting in a large amount of context switch cost.

### Batch

For each Partition, it is necessary to do batch to improve throughput when writing WAL serially. In general, there is nothing special about batch, but Nebula designs some special types of WAL based on each part serialization, which brings some challenges.

For example, Nebula uses WAL to implement lock-free CAS operations, and each CAS operation will execute after all the previous WAL are committed . So for a batch, if there are several WALs in CAS type, we need to divide this batch into several smaller serial groups.

### Learner

New machines need to "catch up" data for quite a long time, and there may be accidents during the process. If a new machine join the raft group as a follower, it will reduce the HA capacity of the entire cluster. Nebula uses the command WAL mentioned above to implement its learner. If the leader encounters the `add learner command` when writing WAL, it will add the learner to its peers and mark it as a learner. So logs will send to them as normal. But the learners will not evolve in the leader's election.

### Transfer Leadership

Transfer leadership is extremely important for balance. When moving a partition from one machine to another, Nebula first checks if the source is a leader. If so, it should be moved to another peer. After  data migration is completed, please do balance the leader again, so that the load on each machine can be balanced.

When transferring leadership, note that when the leader abandons its leadership and when the follower starts a leader election. When a transfer leadership command is committed, the original leader abandons its leadership. And as for followers, when receiving the command, it starts a leader election. This is the same execution process with common raft leader election. Otherwise some corner cases will occur.

### Membership change

To avoid split-brain, when members in a Raft Group change, an intermediate state is required. In such state, the majority of the old group and  new group always have an overlap, thus preventing the old or new group from making decision unilaterally. This is the `joint consensus` mentioned in the thesis. To make it even simpler, Diego Ongaro suggests **adding or removing a peer once to ensure the the overlap between the majority of the new group and the old group** in his doctoral thesis. Nebula's implementation also uses this approach, except that the way to add or remove member is different. Details please refer `addPeer/removePeer` in Raft Part class.

### Snapshot

When the leader changes during sending snapshot, the follower may only receive half of the snapshot data. So partition data cleanup is required. Because multiple partitions share a single storage, how to clean up the data is a cumbersome problem. In addition, during the snapshot process, a large amount of IO is generated. Considering the performance, we do not want this process to share an IO threadPool with the normal Raft, and the process requires a large amount of memory, thus how to optimize the memory usage is critical for performance.

## Storage Service

- `getNeighbors`: query in in-edge or out-edge of a set of vertices, return the edges and the corresponding properties, support conditional filters
- `Insert vertex/edge`: insert a vertex or edge and its properties
- `getProps`: get the properties of a vertex or an edge

Graph semantics interfaces are translated into kv operations in this layer. In order to improve the performance of traversal, concurrent operations are also required.

## Meta Service

On the kv store interface, Nebula encapsulates a set of meta-related interfaces. Meta service supports CRUD on schema, cluster management and user management. Meta service supports single-machine deployment, as well as multiple replicas to ensure data security.
